@@ -25,13 +25,6 @@ class Gem::Commands::BuildCommand < Gem::Command
     add_option "-o", "--output FILE", "output gem with the given filename" do |value, options|
       options[:output] = value
     end
-
-    add_option "-C PATH", "Run as if gem build was started in <PATH> instead of the current working directory." do |value, options|
-      options[:build_path] = value
-    end
-    deprecate_option "-C",
-                     version: "4.0",
-                     extra_msg: "-C is a global flag now. Use `gem -C PATH build GEMSPEC_FILE [options]` instead"
   end
 
   def arguments # :nodoc:
