@@ -1,27 +1,15 @@
-require "minitest/test"
+require_relative "test"
 
 class Module # :nodoc:
   def infect_an_assertion meth, new_name, dont_flip = false # :nodoc:
     block = dont_flip == :block
     dont_flip = false if block
-    target_obj = block ? '_{obj.method}' : '_(obj)'
 
     # https://eregon.me/blog/2021/02/13/correct-delegation-in-ruby-2-27-3.html
     # Drop this when we can drop ruby 2.6 (aka after rails 6.1 EOL, ~2024-06)
-    kw_extra = "ruby2_keywords %p" % [new_name] if respond_to?(:ruby2_keywords, true)
+    kw_extra = "ruby2_keywords %p" % [new_name] if respond_to? :ruby2_keywords, true
 
-    # warn "%-22p -> %p %p" % [meth, new_name, dont_flip]
     self.class_eval <<-EOM, __FILE__, __LINE__ + 1
-      def #{new_name} *args
-        where = Minitest.filter_backtrace(caller).first
-        where = where.split(/:in /, 2).first # clean up noise
-        Kernel.warn "DEPRECATED: global use of #{new_name} from #\{where}. Use #{target_obj}.#{new_name} instead. This will fail in Minitest 6."
-        Minitest::Expectation.new(self, Minitest::Spec.current).#{new_name}(*args)
-      end
-      #{kw_extra}
-    EOM
-
-    Minitest::Expectation.class_eval <<-EOM, __FILE__, __LINE__ + 1
       def #{new_name} *args
         raise "Calling ##{new_name} outside of test." unless ctx
         case
@@ -38,7 +26,11 @@ class Module # :nodoc:
   end
 end
 
-Minitest::Expectation = Struct.new :target, :ctx # :nodoc:
+# :stopdoc:
+module Minitest # fucking hell rdoc...
+  Expectation = Struct.new :target, :ctx
+end
+# :startdoc:
 
 ##
 # Kernel extensions for minitest
@@ -81,14 +73,15 @@ module Kernel
 
   def describe desc, *additional_desc, &block # :doc:
     stack = Minitest::Spec.describe_stack
-    name  = [stack.last, desc, *additional_desc].compact.join("::")
-    sclas = stack.last || if Class === self && kind_of?(Minitest::Spec::DSL) then
-                            self
-                          else
-                            Minitest::Spec.spec_type desc, *additional_desc
-                          end
+    is_spec_class = Class === self && kind_of?(Minitest::Spec::DSL)
+    name  = [stack.last, desc, *additional_desc]
+    name.prepend self if stack.empty? && is_spec_class
+    sclas =
+      stack.last                 \
+      || (is_spec_class && self) \
+      || Minitest::Spec.spec_type(desc, *additional_desc)
 
-    cls = sclas.create name, desc
+    cls = sclas.create name.compact.join("::"), desc
 
     stack.push cls
     cls.class_eval(&block)
@@ -105,15 +98,6 @@ end
 
 class Minitest::Spec < Minitest::Test
 
-  def self.current # :nodoc:
-    Thread.current[:current_spec]
-  end
-
-  def initialize name # :nodoc:
-    super
-    Thread.current[:current_spec] = self
-  end
-
   ##
   # Oh look! A Minitest::Spec::DSL module! Eat your heart out DHH.
 
@@ -249,7 +233,7 @@ class Minitest::Spec < Minitest::Test
       pre, post = "let '#{name}' cannot ", ". Please use another name."
       methods = Minitest::Spec.instance_methods.map(&:to_s) - %w[subject]
       raise ArgumentError, "#{pre}begin with 'test'#{post}" if
-        name =~ /\Atest/
+        name.start_with? "test"
       raise ArgumentError, "#{pre}override a method in Minitest::Spec#{post}" if
         methods.include? name
 
@@ -268,7 +252,7 @@ class Minitest::Spec < Minitest::Test
     end
 
     def create name, desc # :nodoc:
-      cls = Class.new(self) do
+      cls = Class.new self do
         @name = name
         @desc = desc
 
@@ -284,12 +268,11 @@ class Minitest::Spec < Minitest::Test
       defined?(@name) ? @name : super
     end
 
-    def to_s # :nodoc:
-      name # Can't alias due to 1.8.7, not sure why
-    end
+    alias to_s name
+    alias inspect name
 
     attr_reader :desc # :nodoc:
-    alias :specify :it
+    alias specify it
 
     ##
     # Rdoc... why are you so dumb?
@@ -309,9 +292,6 @@ class Minitest::Spec < Minitest::Test
       # straight-expectation methods (on Object) because it stores its
       # test context, bypassing our hacky use of thread-local variables.
       #
-      # NOTE: At some point, the methods on Object will be deprecated
-      # and then removed.
-      #
       # It is also aliased to #value and #expect for your aesthetic
       # pleasure:
       #
@@ -325,11 +305,6 @@ class Minitest::Spec < Minitest::Test
 
       alias value _
       alias expect _
-
-      def before_setup # :nodoc:
-        super
-        Thread.current[:current_spec] = self
-      end
     end
 
     def self.extended obj # :nodoc:
@@ -342,8 +317,8 @@ class Minitest::Spec < Minitest::Test
   TYPES = DSL::TYPES # :nodoc:
 end
 
-require "minitest/expectations"
+require_relative "expectations"
 
-class Object # :nodoc:
-  include Minitest::Expectations unless ENV["MT_NO_EXPECTATIONS"]
+class Minitest::Expectation
+  include Minitest::Expectations
 end
