@@ -104,6 +104,18 @@ module RBS
           @comment = comment
         end
 
+        def update(name: self.name, type_params: self.type_params, super_class: self.super_class, members: self.members, annotations: self.annotations, location: self.location, comment: self.comment)
+          self.class.new(
+            name: name,
+            type_params: type_params,
+            super_class: super_class,
+            members: members,
+            annotations: annotations,
+            location: location,
+            comment: comment
+          )
+        end
+
         def ==(other)
           other.is_a?(Class) &&
             other.name == name &&
@@ -192,6 +204,19 @@ module RBS
           @comment = comment
         end
 
+        def update(name: self.name, type_params: self.type_params, members: self.members, self_types: self.self_types, annotations: self.annotations, location: self.location, comment: self.comment)
+          self.class.new(
+            name: name,
+            type_params: type_params,
+            members: members,
+            self_types: self_types,
+            annotations: annotations,
+            location: location,
+            comment: comment
+          )
+        end
+
+
         def ==(other)
           other.is_a?(Module) &&
             other.name == name &&
@@ -239,6 +264,17 @@ module RBS
           @comment = comment
         end
 
+        def update(name: self.name, type_params: self.type_params, members: self.members, annotations: self.annotations, location: self.location, comment: self.comment)
+          self.class.new(
+            name: name,
+            type_params: type_params,
+            members: members,
+            annotations: annotations,
+            location: location,
+            comment: comment
+          )
+        end
+
         def ==(other)
           other.is_a?(Interface) &&
             other.name == name &&
@@ -313,12 +349,14 @@ module RBS
         attr_reader :type
         attr_reader :location
         attr_reader :comment
+        attr_reader :annotations
 
-        def initialize(name:, type:, location:, comment:)
+        def initialize(name:, type:, location:, comment:, annotations: [])
           @name = name
           @type = type
           @location = location
           @comment = comment
+          @annotations = annotations || []
         end
 
         def ==(other)
@@ -349,12 +387,14 @@ module RBS
         attr_reader :type
         attr_reader :location
         attr_reader :comment
+        attr_reader :annotations
 
-        def initialize(name:, type:, location:, comment:)
+        def initialize(name:, type:, location:, comment:, annotations: [])
           @name = name
           @type = type
           @location = location
           @comment = comment
+          @annotations = annotations
         end
 
         def ==(other)
@@ -381,13 +421,14 @@ module RBS
       end
 
       class AliasDecl < Base
-        attr_reader :new_name, :old_name, :location, :comment
+        attr_reader :new_name, :old_name, :location, :comment, :annotations
 
-        def initialize(new_name:, old_name:, location:, comment:)
+        def initialize(new_name:, old_name:, location:, comment:, annotations: [])
           @new_name = new_name
           @old_name = old_name
           @location = location
           @comment = comment
+          @annotations = annotations
         end
 
         def ==(other)
