@@ -58,6 +58,7 @@ module TypeProf::Core
       when :begin_node then BeginNode.new(raw_node, lenv)
       when :retry_node then RetryNode.new(raw_node, lenv)
       when :rescue_modifier_node then RescueModifierNode.new(raw_node, lenv)
+      when :rescue_node then RescueNode.new(raw_node, lenv)
 
       # constants
       when :constant_read_node, :constant_path_node
@@ -92,6 +93,8 @@ module TypeProf::Core
       # variables
       when :local_variable_read_node
         LocalVariableReadNode.new(raw_node, lenv)
+      when :it_local_variable_read_node
+        ItLocalVariableReadNode.new(raw_node, lenv)
       when :local_variable_write_node
         LocalVariableWriteNode.new(raw_node, AST.create_node(raw_node.value, lenv), lenv)
       when :local_variable_operator_write_node
@@ -223,7 +226,7 @@ module TypeProf::Core
       when :flip_flop_node then FlipFlopNode.new(raw_node, lenv)
       when :shareable_constant_node then create_node(raw_node.write, lenv)
       when :match_required_node then MatchRequiredNode.new(raw_node, lenv)
-      when :match_predicate_node then MatchPreidcateNode.new(raw_node, lenv)
+      when :match_predicate_node then MatchPredicateNode.new(raw_node, lenv)
 
       # call
       when :super_node then SuperNode.new(raw_node, lenv)
@@ -267,6 +270,8 @@ module TypeProf::Core
         IndexWriteNode.new(raw_node, dummy_node, lenv)
       when :call_target_node
         CallWriteNode.new(raw_node, dummy_node, lenv)
+      when :multi_target_node
+        MultiTargetNode.new(raw_node, dummy_node, lenv)
       else
         pp raw_node
         raise "not supported yet: #{ raw_node.type }"
@@ -343,8 +348,7 @@ module TypeProf::Core
           break
         when :constant_path_node, :constant_path_target_node
           if raw_node.parent
-            # temporarily support old Prism https://bugs.ruby-lang.org/issues/20467
-            names << (raw_node.respond_to?(:name) ? raw_node.name : raw_node.child.name)
+            names << raw_node.name
             raw_node = raw_node.parent
           else
             return names.reverse
@@ -356,7 +360,8 @@ module TypeProf::Core
           return nil
         end
       end
-      return cref.cpath + names.reverse
+      cpath = cref.cpath # annotate
+      return cpath + names.reverse if cpath
     end
 
     def self.parse_rbs(path, src)
@@ -397,6 +402,8 @@ module TypeProf::Core
         SigDefNode.new(raw_decl, lenv)
       when RBS::AST::Members::Include
         SigIncludeNode.new(raw_decl, lenv)
+      when RBS::AST::Members::Prepend
+        SigPrependNode.new(raw_decl, lenv)
       when RBS::AST::Members::Extend
       when RBS::AST::Members::Public
       when RBS::AST::Members::Private
@@ -410,6 +417,12 @@ module TypeProf::Core
         SigAttrAccessorNode.new(raw_decl, lenv)
       when RBS::AST::Declarations::Base
         self.create_rbs_decl(raw_decl, lenv)
+      when RBS::AST::Members::InstanceVariable
+        SigInstanceVariableNode.new(raw_decl, lenv, false)
+      when RBS::AST::Members::ClassInstanceVariable
+        SigInstanceVariableNode.new(raw_decl, lenv, true)
+      when RBS::AST::Members::ClassVariable
+        SigClassVariableNode.new(raw_decl, lenv)
       else
         raise "unsupported: #{ raw_decl.class }"
       end
